﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace howto_edit_picture_list2
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The currently loaded pictures.
        private List<Bitmap> Pictures = new List<Bitmap>();
        private const int PictureMargin = 8;

        // The index of the picture we clicked or
        // the picture before which we clicked.
        private int ClickedIndex = -1;

        private void ArrangePanel()
        {
            panPictures.Controls.Clear();
            int x = PictureMargin;
            int y = PictureMargin;
            foreach (Bitmap picture in Pictures)
            {
                PictureBox pic = new PictureBox();
                pic.SizeMode = PictureBoxSizeMode.AutoSize;
                pic.Location = new Point(x, y);
                pic.Image = picture;
                pic.Visible = true;
                pic.BorderStyle = BorderStyle.Fixed3D;
                pic.MouseDown += pic_MouseDown;
                panPictures.Controls.Add(pic);

                x += pic.Width + PictureMargin;
            }

            // Add one placeholder PictureBox.
            PictureBox placeholder = new PictureBox();
            placeholder.Location = new Point(x, y);
            placeholder.Size = new Size(0, 0);
            placeholder.Visible = true;
            placeholder.MouseDown += pic_MouseDown;
            panPictures.Controls.Add(placeholder);
        }

        private void pic_MouseDown(object sender, MouseEventArgs e)
        {
            // Ignore left mouse clicks.
            if (e.Button != MouseButtons.Right) return;

            PictureBox pic = sender as PictureBox;

            // Get the mouse's location in panPictures coordinates.
            Point screen_point = pic.PointToScreen(e.Location);
            Point parent_point = panPictures.PointToClient(screen_point);

            // Display the context menu.
            ShowContextMenu(new Point(
                parent_point.X,
                parent_point.Y));
        }

        private void panPictures_MouseDown(object sender, MouseEventArgs e)
        {
            // Ignore left mouse clicks.
            if (e.Button != MouseButtons.Right) return;

            // Display the context menu.
            ShowContextMenu(e.Location);
        }

        private void mnuMoveLeft_Click(object sender, EventArgs e)
        {
            Bitmap bm = Pictures[ClickedIndex];
            Pictures.RemoveAt(ClickedIndex);
            Pictures.Insert(ClickedIndex - 1, bm);
            ArrangePanel();
        }

        private void mnuMoveRight_Click(object sender, EventArgs e)
        {
            Bitmap bm = Pictures[ClickedIndex];
            Pictures.RemoveAt(ClickedIndex);
            Pictures.Insert(ClickedIndex + 1, bm);
            ArrangePanel();
        }

        private void mnuDeletePicture_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(
                "Are you sure you want to delete this picture?",
                "Delete Picture?", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                Pictures.RemoveAt(ClickedIndex);
                ArrangePanel();
            }
        }

        // Let the user insert a picture.
        private void mnuInsertPicture_Click(object sender, EventArgs e)
        {
            try
            {
                if (ofdPicture.ShowDialog() == DialogResult.OK)
                {
                    int i = 0;
                    foreach (string filename in ofdPicture.FileNames)
                    {
                        Bitmap bm = new Bitmap(filename);
                        Pictures.Insert(ClickedIndex + i, bm);
                        i++;
                    }
                    ArrangePanel();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        // Prepare the context menu and display it.
        private void ShowContextMenu(Point location)
        {
            // Assume we click after the final picture.
            bool clicked_on_picture = false;
            ClickedIndex = Pictures.Count;

            // See if we clicked on or before a picture.
            int x = location.X + panPictures.HorizontalScroll.Value;
            for (int i = 0; i < Pictures.Count; i++)
            {
                // See if we are before the next picture.
                x -= PictureMargin;
                if (x < 0)
                {
                    ClickedIndex = i;
                    break;
                }   

                // See if we are on this picture.
                x -= panPictures.Controls[i].Width;
                if (x < 0)
                {
                    ClickedIndex = i;
                    clicked_on_picture = true;
                    break;
                }
            }

            // Enable and disable contect menu items.
            mnuMoveLeft.Enabled =
                (clicked_on_picture && (ClickedIndex > 0));
            mnuMoveRight.Enabled =
                (clicked_on_picture && (ClickedIndex < Pictures.Count - 1));
            mnuDeletePicture.Enabled = clicked_on_picture;
            mnuInsertPicture.Enabled = !clicked_on_picture;

            // Display the context menu.
            ctxPictures.Show(panPictures, location);
        }
    }
}
